-- Title   : Put String at XY position (PUTSXY)
-- Author  : David Alan Gay
-- Created : Jan 7/97
-- Purpose : To allow the display of text at a specific pixel location,
--           with a specified foreground and background color. The text
--           can be displayed left to right, or top to bottom.
-- Modified: Jan 7/98 Rob Craig - made it faster and simpler, since a lot of 
--           people are using it. 
-- Modified: Sep 7/98 Rob Craig - it now uses save_image() to pick up the 
--           fonts, for characters 0 to 255. This makes it 2000 lines shorter. 
--           Note: You must first call init_putsxy() to initialize font_index.
-- Modified: Sep 20/98 Rob Craig - you no longer need a BLACK background
--           for init_putsxy() to work
-- Modified: Sep 20/98 Rob Craig - putsxy() is now about 4x faster for
--           typical strings. Rather than calling display_image() for each 
--           character in a string, putsxy() now builds a complete image of the 
--           whole string, and then makes a single call to display_image().

include image.e

sequence font_index
integer background_color

global procedure init_putsxy()
-- Call this to initialize the fonts.
-- You must be in a pixel graphics mode, such as 18.
    font_index = repeat(repeat(repeat(0,8),16),256)
    for i = 0 to 255 do
	position(1,1)
	puts(1, ' ')  -- needed to clear '$' top pixel - strange BIOS bug
	position(2,1)
	puts(1, i)
	font_index[i+1] = save_image({0,16}, {7,31}) -- 16x8 font
    end for
    background_color = get_pixel({0,15})
end procedure

global procedure putsxy(sequence pixel_loc, sequence display_text,
			atom foreground, atom background, atom direction)
-- Display text at an {x,y} pixel location, using the specified foreground
-- and background colors. If direction is 'd' then text is printed
-- down the screen, otherwise it is printed across the screen.
    integer x, y, nchars, string_row, string_col
    sequence font_image, font_row, string_image
    
    nchars = length(display_text)
    x = pixel_loc[1]
    y = pixel_loc[2]
    if direction = 'd' or direction = 'D' then
	-- printing goes down the screen
	string_image = repeat(repeat(background, 8), nchars*16)
	string_row = 1
	for ix = 1 to nchars do
	    font_image = font_index[1+display_text[ix]]
	    -- copy the character into the string image
	    for row = 1 to 16 do
		font_row = font_image[row]
		for col = 1 to 8 do
		    if font_row[col] != background_color then
			string_image[string_row][col] = foreground
		    end if
		end for
		string_row = string_row + 1
	    end for
	end for
    else            
	-- printing goes across the screen
	string_image = repeat(repeat(background, nchars*8),16)
	string_col = 0
	for ix = 1 to nchars do
	    font_image = font_index[1+display_text[ix]]
	    -- copy the character into the string image
	    for row = 1 to 16 do
		font_row = font_image[row]
		for col = 1 to 8 do
		    if font_row[col] != background_color then
			string_image[row][string_col+col] = foreground
		    end if
		end for
	    end for
	    string_col = string_col + 8
	end for
    end if
    display_image({x, y}, string_image)
end procedure

